from PIL import Image
import os
from  utils.conf import *
from tos import HttpMethodType
import time

def get_lm_text(sys_prompt,user_prompt):
    completion = LMConfig.lm_client.chat.completions.create(
        messages = [
            {"role": "system", "content": sys_prompt},
            {"role": "user", "content": user_prompt},
        ],
        model=LMConfig.model,
    )

    return completion.choices[0].message.content




## 多模态的输入
def compress_image(input_path, output_path):
    img = Image.open(input_path)
    current_size = os.path.getsize(input_path)
    # 粗略的估计压缩质量，也可以从常量开始，逐步减小压缩质量，直到文件大小小于目标大小
    image_quality = int(float(MMMConfig.target_size / current_size) * 100)
    img.save(output_path, optimize=True, quality=int(float(MMMConfig.target_size / current_size) * 100))
    # 如果压缩后文件大小仍然大于目标大小，则继续压缩
    # 压缩质量递减，直到文件大小小于目标大小
    while os.path.getsize(output_path) > MMMConfig.target_size:
        img = Image.open(output_path)
        image_quality -= 10
        if image_quality <= 0:
            break
        img.save(output_path, optimize=True, quality=image_quality)
    return image_quality

def upload_tos(filename,  tos_object_key):

    tos_client, inner_tos_client = MMMConfig.tos_client, MMMConfig.inner_tos_client
    try:
        # 将本地文件上传到目标桶中, filename为本地压缩后图片的完整路径
        tos_client.put_object_from_file(MMMConfig.tos_bucket_name, tos_object_key, filename)
        # 获取上传后预签名的 url
        return inner_tos_client.pre_signed_url(HttpMethodType.Http_Method_Get, MMMConfig.tos_bucket_name, tos_object_key)
    except Exception as e:
        if isinstance(e, tos.exceptions.TosClientError):
            # 操作失败，捕获客户端异常，一般情况为非法请求参数或网络异常
            print('fail with client error, message:{}, cause: {}'.format(e.message, e.cause))
        elif isinstance(e, tos.exceptions.TosServerError):
            # 操作失败，捕获服务端异常，可从返回信息中获取详细错误信息
            print('fail with server error, code: {}'.format(e.code))
            # request id 可定位具体问题，强烈建议日志中保存
            print('error with request id: {}'.format(e.request_id))
            print('error with message: {}'.format(e.message))
            print('error with http code: {}'.format(e.status_code))

        else:

            print('fail with unknown error: {}'.format(e))

        raise e



def doubao_MMM_request(pre_signed_url_output, prompt):

    client = MMMConfig.client
    

    response = client.chat.completions.create(
        model=MMMConfig.model,
        messages=[{"role": "user","content": [
                {"type": "text", "text": prompt},
                {"type": "image_url", "image_url": {"url": pre_signed_url_output.signed_url}}
            ],
        }],
        temperature=0.8,
        extra_headers={"x-ark-beta-vision": "true"}
    )
    result = response.choices[0].message.content
    return result